#!/usr/bin/env python
# encoding: utf-8
"""
world.py - countries of the world (to be procise: countries we do buiseness with)

Created by Maximillian Dornseif on 2007-05-01.
Copyright (c) 2007 HUDORA GmbH. BSD Licensed.
"""

__revision__ = "$Revision: 3951 $"

COUNTRY_CHOICES = [('DE', 'Deutschland')] \
                   + sorted([
                   ('AT', 'Östereich'),
                   ('CH', 'Schweiz'),
                   ('BE', 'Belgien'),
                   ('FR', 'Frankreich'),
                   ('ES', 'Spanien'),
                   ('NL', 'Niederlande'),
                   ('IT', 'Italien'),
                   ('GB', 'Grossbritannien'),
                   ('LI', 'Lichtenstein'),
                   ('LU', 'Luxenburg'),
                   ('PT', 'Portugal'),
                   ('CZ', 'Tschechien'),
                   ('SE', 'Schweden'),
                   ('GR', 'Griechenland'),
                   ('DK', 'Daenemark'),
                   ('US', 'USA'),
                   ('AR', 'Argentinien'),
                   ('HU', 'Ungarn'),
                   ('LV', 'Lettland'),
                   ('PL', 'Polen'),
                   ('LT', 'Littauen'),
                   ('SK', 'Slowakei'),
                   ('HR', 'Kroatioen'),
                   ('IE', 'Irland'),
                   ('SI', 'Slowenien'),
                   ('EE', 'Estland'),
                   ('BG', 'Bulgarien'),
                   ('NO', 'Norwegen'),
                   ('AU', 'Australien'),
                  ])
